/*This program estimates our lower and upper bounds, as well as IPV profit curves
for the parametric reference model. These were used in the bandwidth selection procedure
described in the online appendix*/

dinit=date;

grid_z=20000;
simul_Z=rndn(grid_z,1);


library maxlik;
maxset;
__output=0;
_max_MaxIters=2000;


betas_matrix=zeros(13,10);
betas2_matrix=zeros(13,10);
n_bar=11;


/*To be used to compute phi (through a simple grid search)*/
grid_size=20000;
aux_grid=zeros(grid_size-1,1);
i=1;
do while i .le grid_size-1;
aux_grid[i]=i/grid_size;
i=i+1;
endo;

conditional_X_only=0; /*1 if profits are conditional on X only, 0 if they are also conditional on N*/
n_value=2; /*if profits are conditional on N too, n_value is the conditioning value of N*/

cuantil=0.50; /*Expected profit curves are computed conditional on X=X^(cuantil)*/
xx_value=quantile(saleval_original,cuantil)~quantile(mfgcost_original~harvcost_original,1-cuantil)~quantile(species_conc_original,cuantil)~quantile(inventory_original,1-cuantil)~quantile(apprice_original,cuantil);
xx_value=xx_value';
xx_value_plus_constant=1|xx_value;
vv_0=ceil(quantile(apprice,cuantil));


L=1113; /*Sample size*/
load datos_original[L,9]=data_file_auctions.asc;

B_original=datos_original[.,2];

datos_original=selif(datos_original, B_original .le 650);
L_original=rows(datos_original);


aux_vector_n=2|3|4|5|6|7|8|9|10|11;
aux_vector_Q_a=
0|(1/(3*(3-1)))|(1/(4*(4-1)))|(1/(5*(5-1)))|(1/(6*(6-1)))|(1/(7*(7-1)))|(1/(8*(8-1)))|(1/(9*(9-1)))|(1/(10*(10-1)))|(1/(11*(11-1)));

N_original=datos_original[.,1];
B_original=datos_original[.,2];
saleval_original=datos_original[.,3]/stdc(datos_original[.,3]);
mfgcost_original=datos_original[.,4]/stdc(datos_original[.,4]);
harvcost_original=datos_original[.,5]/stdc(datos_original[.,5]);
species_conc_original=datos_original[.,6]/stdc(datos_original[.,6]);
inventory_original=datos_original[.,7]/stdc(datos_original[.,7]);
apprice_original=datos_original[.,8]/stdc(datos_original[.,8]);
tvol_original=datos_original[.,9];
apprice=datos_original[.,8];

variables_original=saleval_original~mfgcost_original~harvcost_original~species_conc_original~inventory_original~apprice_original;

variables_original_plus_constant=ones(L_original,1)~variables_original;

projection_original=pinv(variables_original_plus_constant'*variables_original_plus_constant)*variables_original_plus_constant';

covariates_original=ones(L_original,1)~N_original~variables_original~N_original.*variables_original;

covariates_original_variance=covariates_original;


betas_E_N_m_X_matrix=projection_original*(N_original.*(N_original .lt aux_vector_n'));
betas_E_N_X=projection_original*N_original;

proc loglikelih(bb,z);
local theta_mu, theta_sigma, index_mu, index_sigma;
local log_B, dens, sigma_2, zet;

log_B=ln(B_original);
theta_mu=bb[1:cols(covariates_original)];
theta_sigma=bb[cols(covariates_original)+1:cols(covariates_original)+cols(covariates_original_variance)];

index_mu=covariates_original*theta_mu;
index_sigma=sqrt(exp(covariates_original_variance*theta_sigma));

sigma_2=index_sigma.*index_sigma;

zet=(log_B-index_mu)./(sqrt(2)*(index_sigma));

dens=(1/B_original).*(1/sqrt(2*pi*sigma_2)).*exp(-zet.*zet);

retp(ln(dens));
endp;


bb0=zeros(cols(covariates_original),1)|ones(cols(covariates_original_variance),1)*1;
{beta_mle,f_mle,g_mle,cov_mle,ret_mle}=maxlik("aux_mle", 0, &loglikelih, bb0);

beta_mle_mu=beta_mle[1:cols(covariates_original)];

beta_mle_sigma=beta_mle[cols(covariates_original)+1:2*cols(covariates_original)];


proc loglikel(bb,z);
local indice_n, N_indicador, theta, covars_probabs;
local exp_covars, sum_exp_covars, indices_probabs, log_likel;
indice_n=seqa(2,1,9);
theta=reshape(bb,cols(variables_original)+1,rows(indice_n));
covars_probabs=(ones(L_original,1)~variables_original)*theta;
exp_covars=exp(covars_probabs);
sum_exp_covars=sumc(exp_covars');
indices_probabs=(exp_covars~ones(L_original,1))./(1+sum_exp_covars);
N_indicador=(N_original .eq aux_vector_n');
log_likel=sumc((N_indicador.*ln(indices_probabs))');
retp(log_likel);
endp;

bb0=zeros((cols(variables_original)+1)*9,1);
{xv,fff,ggg,cov,ret}=maxlik("aux_mle", 0, &loglikel, bb0);
probab_thetas_new=reshape(xv,cols(variables_original)+1,9);


reserve_price=selif(B_original, (B_original .ge vv_0).*(B_original .le 400)); /*Range of reserve prices at which profits will be evaluated*/

gridsize=rows(reserve_price);

F_n_1_n_matrix_mle=zeros(gridsize,10);
phi_n_1_n_matrix_mle=zeros(gridsize,10);
T_r_x_param=zeros(gridsize,1);
prob_nosale_low_mle=zeros(gridsize,1);
prob_nosale_up_mle=zeros(gridsize,1);
prob_nosale_ipv_mle=zeros(gridsize,1);
pi_low_parametric=zeros(gridsize,1);
pi_up_parametric=zeros(gridsize,1);
pi_ipv_parametric=zeros(gridsize,1);

if conditional_X_only .eq 1;
E_N_m_X_xx_value=sumc(xx_value_plus_constant.*betas_E_N_m_X_matrix);
E_N_X=sumc(xx_value_plus_constant.*betas_E_N_X);
else;
E_N_m_X_xx_value=n_value.*(n_value .lt aux_vector_n);
E_N_X=n_value;
endif;


prob_covars=(1~xx_value')*probab_thetas_new;
exp_prob_covars=exp(prob_covars);
sum_exp_prob_covars=sumc(exp_prob_covars');
indices_probab=(exp_prob_covars~1)./(1+sum_exp_prob_covars);

if conditional_X_only .eq 1;
P_N_X=indices_probab';
else;
P_N_X=(aux_vector_n .eq n_value);
endif;

mu_x_n_mle_vector=zeros(10,1);
sigma_x_n_mle_vector=zeros(10,1);

rr=1;
do while rr .le gridsize;
ene=2;
do while ene .le n_bar;
nn_value=ene;
covars_xx_value=1|nn_value|xx_value|nn_value.*xx_value;
mu_x_n_mle=sumc(covars_xx_value.*beta_mle_mu);
sigma_x_n_mle=sqrt(exp(sumc(covars_xx_value.*beta_mle_sigma)));
mu_x_n_mle_vector[ene-1]=mu_x_n_mle;
sigma_x_n_mle_vector[ene-1]=sigma_x_n_mle;
F_n_1_n_matrix_mle[rr,ene-1]=cdfn((ln(reserve_price[rr])-mu_x_n_mle)./sigma_x_n_mle);
aux_phi=sortc(aux_grid~(F_n_1_n_matrix_mle[rr,ene-1]-(nn_value*aux_grid^(nn_value-1)-(nn_value-1)*aux_grid^(nn_value)))^2,2);
phi_n_1_n_matrix_mle[rr,ene-1]=aux_phi[1,1];
ene=ene+1;
endo;

sigma_x_n_2_mle_vector=sigma_x_n_mle_vector^2;

mu_x_mle=sumc(mu_x_n_mle_vector.*P_N_X);

mu_x_2_mle=sumc((mu_x_n_mle_vector^2).*P_N_X);

var_mu_x_mle=mu_x_2_mle-mu_x_mle^2;

E_sigma_x_n_2_mle_vector=sumc(sigma_x_n_2_mle_vector.*P_N_X);

var_x_mle=var_mu_x_mle+E_sigma_x_n_2_mle_vector;

B_simul=exp(sqrt(var_x_mle)*simul_Z+mu_x_mle);

T_r_x_param[rr]=meanc(maxc((B_simul~ones(grid_z,1)*reserve_price[rr])'));

prob_nosale_low_mle[rr]=F_n_1_n_matrix_mle[rr,.]*(aux_vector_Q_a.*E_N_m_X_xx_value)+(E_N_X/n_bar)*(phi_n_1_n_matrix_mle[rr,10])^(n_bar);

prob_nosale_up_mle[rr]=F_n_1_n_matrix_mle[rr,.]*(aux_vector_Q_a.*E_N_m_X_xx_value)+(E_N_X/n_bar)*F_n_1_n_matrix_mle[rr,10];

prob_nosale_ipv_mle[rr]=(phi_n_1_n_matrix_mle[rr,.].^(aux_vector_n'))*P_N_X;

pi_low_parametric[rr]=T_r_x_param[rr]-vv_0-prob_nosale_up_mle[rr].*(reserve_price[rr]-vv_0);
pi_up_parametric[rr]=T_r_x_param[rr]-vv_0-prob_nosale_low_mle[rr].*(reserve_price[rr]-vv_0);
pi_ipv_parametric[rr]=T_r_x_param[rr]-vv_0-prob_nosale_ipv_mle[rr].*(reserve_price[rr]-vv_0);


rr=rr+1;
endo;

dfin=date;
if dfin[3] .eq dinit[3];
totaltime=(dfin[4]-dinit[4])/100;
else;

totaltime=(8640000*(dfin[3]-dinit[3])-dinit[4]+dfin[4])/100;

endif;
